diff --git a/src/Mods/SecExp/buildings/secBarracks.tw b/src/Mods/SecExp/buildings/secBarracks.tw
index 4a2cfd031e97600452b2b4f06ebd196c69968e19..d1f63a74a1682f7c8aba6f0bc22413160e2b764a 100644
--- a/src/Mods/SecExp/buildings/secBarracks.tw
+++ b/src/Mods/SecExp/buildings/secBarracks.tw
@@ -227,7 +227,7 @@ Your current maximum number of units is <<print App.SecExp.battle.maxUnits()>> (
 				<<set $SecExp.units.slaves.squads.push(App.SecExp.unit.gen("slaves"))>>
 			<</link>> |
 		<</if>>
-		<<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.slaves.squads)>>
+		<<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.slaves.squads, "slaves")>>
 
 		<br> <<includeDOM App.UI.market({menialWorkersOnly: true})>>
 		<<set _sL = $SecExp.units.slaves.squads.length>>
@@ -264,7 +264,7 @@ Your current maximum number of units is <<print App.SecExp.battle.maxUnits()>> (
 					<</link>>
 				<</if>>
 			<</if>>
-			| <<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.slaves.squads[_i])>>
+			| <<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.slaves.squads[_i], "slaves")>>
 			<<if $SecExp.settings.showStats == 1>> <<= App.SecExp.getUnit("slaves", _i).printStats()>> <</if>>
 			<<includeDOM App.SecExp.unit.humanUpgradeList($SecExp.units.slaves.squads[_i])>>
 			<</capture>>
@@ -290,7 +290,7 @@ Your current maximum number of units is <<print App.SecExp.battle.maxUnits()>> (
 				<<set $SecExp.units.militia.squads.push(App.SecExp.unit.gen("militia"))>>
 			<</link>> |
 		<</if>>
-		<<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.militia.squads)>>
+		<<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.militia.squads, "militia")>>
 
 		<br>
 		<<set _mL = $SecExp.units.militia.squads.length>>
@@ -327,7 +327,7 @@ Your current maximum number of units is <<print App.SecExp.battle.maxUnits()>> (
 					<</link>>
 				<</if>>
 			<</if>>
-			| <<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.militia.squads[_i])>>
+			| <<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.militia.squads[_i], "militia")>>
 			<<if $SecExp.settings.showStats == 1>> <<= App.SecExp.getUnit("militia", _i).printStats()>> <</if>>
 			<<includeDOM App.SecExp.unit.humanUpgradeList($SecExp.units.militia.squads[_i])>>
 			<</capture>>
@@ -344,7 +344,7 @@ Your current maximum number of units is <<print App.SecExp.battle.maxUnits()>> (
 				<<set $SecExp.units.mercs.squads.push(App.SecExp.unit.gen("mercs"))>>
 			<</link>> |
 		<</if>>
-		<<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.mercs.squads)>>
+		<<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.mercs.squads, "mercs")>>
 
 		<br>
 		<<set _meL = $SecExp.units.mercs.squads.length>>
@@ -381,7 +381,7 @@ Your current maximum number of units is <<print App.SecExp.battle.maxUnits()>> (
 					<</link>>
 				<</if>>
 			<</if>>
-			| <<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.mercs.squads[_i])>>
+			| <<includeDOM App.SecExp.unit.bulkUpgrade($SecExp.units.mercs.squads[_i], "mercs")>>
 			<<if $SecExp.settings.showStats == 1>> <<= App.SecExp.getUnit("mercs", _i).printStats()>> <</if>>
 			<<includeDOM App.SecExp.unit.humanUpgradeList($SecExp.units.mercs.squads[_i])>>
 			<</capture>>
diff --git a/src/Mods/SecExp/events/attackReport.js b/src/Mods/SecExp/events/attackReport.js
index eb580efad45a678de070225e81cc14d784321ad2..ef06c5fb6fa7452625572b60497dd103fc8f4613 100644
--- a/src/Mods/SecExp/events/attackReport.js
+++ b/src/Mods/SecExp/events/attackReport.js
@@ -924,35 +924,34 @@ App.Events.attackReport = function() {
 	function unitsBattleReport() {
 		const el = document.createElement("div");
 		if (V.SecExp.war.losses === 0) {
-			if (App.SecExp.battle.deployedUnits('bots')) {
-				App.UI.DOM.appendNewElement("div", el, `Security Drones: no casualties.`);
-			}
 			if (V.SF.Toggle && V.SF.Active >= 1 && V.SecExp.war.deploySF) {
 				App.UI.DOM.appendNewElement("div", el, `${num(V.SF.ArmySize)} soldiers from ${V.SF.Lower} joined the battle: no casualties suffered`);
 			}
-			const noCasualties = function(units) {
+			const noCasualties = function(units, type) {
 				for (const unit of units) {
-					if (unit.isDeployed === 1) {
-						const r = [`${unit.platoonName}: no casualties.`];
-						unit.battlesFought++;
-						if (unit.training < 100) {
-							if (random(1, 100) > 60) {
-								r.push(`Experience has increased.`);
-								unit.training += random(5, 15) + (majorBattle ? 1 : 0) * random(5, 15);
+					if (App.SecExp.unit.isDeployed(unit)) {
+						const r = [`${type !== 'bots' ? `${unit.platoonName}` : `Security Drones`} : no casualties.`];
+						if (type !== "bots") {
+							unit.battlesFought++;
+							if (unit.training < 100) {
+								if (random(1, 100) > 60) {
+									r.push(`Experience has increased.`);
+									unit.training += random(5, 15) + (majorBattle ? 1 : 0) * random(5, 15);
+								}
 							}
 						}
 						App.Events.addNode(el, r, "div");
 					}
 				}
 			};
-			if (App.SecExp.battle.deployedUnits('militia') >= 1) {
-				noCasualties(V.SecExp.units.militia.squads);
-			}
-			if (App.SecExp.battle.deployedUnits('slaves') >= 1) {
-				noCasualties(V.SecExp.units.slaves.squads);
-			}
-			if (App.SecExp.battle.deployedUnits('mercs') >= 1) {
-				noCasualties(V.SecExp.units.mercs.squads);
+			for (const unitClass of App.SecExp.unit.list()) {
+				if (App.SecExp.battle.deployedUnits(unitClass) >= 1) {
+					if (unitClass !== 'bots') {
+						noCasualties(V.SecExp.units[unitClass].squads, unitClass);
+					} else {
+						noCasualties([V.SecExp.units.bots], unitClass);
+					}
+				}
 			}
 		} else if (V.SecExp.war.losses > 0) {
 			// if the losses are more than zero
@@ -993,31 +992,6 @@ App.Events.attackReport = function() {
 			}
 
 			// assigns the losses and notify the player
-			if (App.SecExp.battle.deployedUnits('bots')) {
-				let loss = lossesList.pluck();
-				loss = Math.clamp(loss, 0, V.SecExp.units.bots.troops);
-				V.SecExp.units.bots.troops -= loss;
-				const r = [`Security drones:`];
-				if (loss <= 0) {
-					r.push(`no casualties`);
-				} else if (loss <= V.SecExp.units.bots.troops * 0.2) {
-					r.push(`light casualties`);
-				} else if (loss <= V.SecExp.units.bots.troops * 0.4) {
-					r.push(`moderate casualties`);
-				} else if (loss <= V.SecExp.units.bots.troops * 0.6) {
-					r.push(`heavy casualties`);
-				} else {
-					r.push(`catastrophic casualties`);
-				}
-				r.push(`suffered.`);
-				if (V.SecExp.units.bots.troops <= 5) {
-					V.SecExp.units.bots.active = 0;
-					r.push(`Unfortunately the losses they took were simply too great, their effective combatants are in so small number you can no longer call them a deployable unit. It will take quite the investment to rebuild them.`);
-				} else if (V.SecExp.units.bots.troops <= 10) {
-					r.push(`The unit has very few operatives left, it risks complete annihilation if deployed again.`);
-				}
-				App.Events.addNode(el, r, "div");
-			}
 			if (V.SF.Toggle && V.SF.Active >= 1 && V.SecExp.war.deploySF) {
 				let loss = lossesList.pluck();
 				loss = Math.clamp(loss, 0, V.SF.ArmySize);
@@ -1037,27 +1011,32 @@ App.Events.attackReport = function() {
 				V.SF.ArmySize -= loss;
 				App.Events.addNode(el, r, "div");
 			}
-			if (App.SecExp.battle.deployedUnits('militia') >= 1) {
-				loopThroughUnits(V.SecExp.units.militia.squads);
-			}
-			if (App.SecExp.battle.deployedUnits('slaves') >= 1) {
-				loopThroughUnits(V.SecExp.units.slaves.squads);
+			for (const unitClass of App.SecExp.unit.list()) {
+				if (App.SecExp.battle.deployedUnits(unitClass) >= 1) {
+					if (unitClass !== 'bots') {
+						loopThroughUnits(V.SecExp.units[unitClass].squads, unitClass);
+					} else {
+						loopThroughUnits([V.SecExp.units.bots], unitClass);
+					}
 			}
-			if (App.SecExp.battle.deployedUnits('mercs') >= 1) {
-				loopThroughUnits(V.SecExp.units.mercs.squads);
 			}
 		} else {
 			App.UI.DOM.appendNewElement("div", el, `Error: losses are a negative number or NaN`, "red");
 		}// closes check for more than zero casualties
 		return el;
 
-		function loopThroughUnits(units) {
+		function loopThroughUnits(units, type) {
 			for (const unit of units) {
-				if (unit.isDeployed === 1) {
-					unit.battlesFought++;
+				if (App.SecExp.unit.isDeployed(unit)) {
 					let loss = lossesList.pluck();
 					loss = Math.clamp(loss, 0, unit.troops);
-					const r = [`${unit.platoonName}:`];
+					if (type !== "bots") {
+						unit.battlesFought++;
+					} else {
+						unit.troops -= loss;
+					}
+
+					const r = [`${type !== "bots" ? `${unit.platoonName}` : `Security drones`}:`];
 					if (loss <= 0) {
 						r.push(`no casualties`);
 					} else if (loss <= unit.troops * 0.2) {
@@ -1070,14 +1049,14 @@ App.Events.attackReport = function() {
 						r.push(`catastrophic casualties`);
 					}
 					r.push(`suffered.`);
+					if (type !== "bots") {
 					const med = Math.round(Math.clamp(loss * unit.medics * 0.25, 1, loss));
 					if (unit.medics === 1 && loss > 0) {
 						r.push(`Some men were saved by their medics.`);
 					}
 					unit.troops -= Math.trunc(Math.clamp(loss - med, 0, unit.maxTroops));
 					V.SecExp.units.militia.dead += Math.trunc(loss - med);
-					if (unit.training < 100) {
-						if (random(1, 100) > 60) {
+						if (unit.training < 100 && random(1, 100) > 60) {
 							r.push(`Experience has increased.`);
 							unit.training += random(5, 15) + (majorBattle ? 1 : 0) * random(5, 15);
 						}
@@ -1085,7 +1064,11 @@ App.Events.attackReport = function() {
 					App.Events.addNode(el, r, "div");
 					if (unit.troops <= 5) {
 						unit.active = 0;
-						App.UI.DOM.appendNewElement("div", el, `Unfortunately the losses they took were simply too great, their effective combatants are in so small number you can no longer call them a deployable unit. The remnants will be sent home honored as veterans or reorganized in a new unit.`);
+						if (type !== "bots") {
+							App.UI.DOM.appendNewElement("div", el, `Unfortunately the losses they took were simply too great, their effective combatants are in so small number you can no longer call them a deployable unit. The remnants will be sent home honored as veterans or reorganized in a new unit.`);
+						} else {
+							App.UI.DOM.appendNewElement("div", el, `Unfortunately the losses they took were simply too great, their effective combatants are in so small number you can no longer call them a deployable unit. It will take quite the investment to rebuild them.`);
+						}
 					} else if (unit.troops <= 10) {
 						App.UI.DOM.appendNewElement("div", el, `The unit has very few operatives left, it risks complete annihilation if deployed again.`);
 					}
diff --git a/src/Mods/SecExp/js/reportingRelatedFunctions.js b/src/Mods/SecExp/js/reportingRelatedFunctions.js
index 26f3d9ec92d9940f162e9a1ade8b1e709a6ddbb5..d6007158bf3fe533faa49b2a31a9e754658386f8 100644
--- a/src/Mods/SecExp/js/reportingRelatedFunctions.js
+++ b/src/Mods/SecExp/js/reportingRelatedFunctions.js
@@ -119,7 +119,7 @@ App.SecExp.updateFacilityDamage = function(facility) {
 
 		V.SecExp.rebellions.repairTime[facility]--;
 		IncreasePCSkills('engineering', 0.1);
-		if (V.SecExp.rebellions.repairTime[facility] === 0) {
+		if (V.SecExp.rebellions.repairTime[facility] <= 0) {
 			delete V.SecExp.rebellions.repairTime[facility];
 		}
 	}
diff --git a/src/Mods/SecExp/js/Unit.js b/src/Mods/SecExp/js/Unit.js
index ecbc97a847b20f433c56f5d5b7b954e4371a048e..c62e0891b0ff35da8ffacc8701eef47fcfe57a3f 100644
--- a/src/Mods/SecExp/js/Unit.js
+++ b/src/Mods/SecExp/js/Unit.js
@@ -1,4 +1,7 @@
 App.SecExp.unit = (function() {
+	const equipUpgradeCost = 250;
+	const secBotsUpgradeCost = 250;
+	const secBotsCost = 500;
 	return {
 		list,
 		bulkUpgrade,
@@ -22,39 +25,55 @@ App.SecExp.unit = (function() {
 
 
 	/** Creates a bulk upgrade link for the unit that is passed.
 	 * @param {object} [unit] the unit to be checked.
 	 */
-	function bulkUpgrade(unit) {
+	function bulkUpgrade(unit, type) {
 		unit = Array.isArray(unit) ? unit : [unit];
 		let el = document.createElement("a");
 
 		function upgradeUnit(x) {
 			x.equip = 3;
-			Object.assign(x, {
-				maxTroops: 50, commissars: 2,
-				cyber: 1, medics: 1
-			});
-			x.SF = (V.SF.Toggle && V.SF.Active >= 1 ? 1 : 0);
+			if (type !== "bots") {
+				Object.assign(x, {
+					maxTroops: 50, commissars: 2,
+					cyber: 1, medics: 1
+				});
+				x.SF = (V.SF.Active >= 1 ? 1 : 0);
+			} else {
+				if (x.maxTroops < 80) {
+					x.maxTroops = 80;
+				} else if (V.SF.Toggle && V.SF.Active >= 1 && x.maxTroops < 100 && V.SecExp.edicts.SFSupportLevel >= 1) {
+					x.maxTroops = 100;
+				}
+			}
 		}
 
 		function getCost(x) {
 			let cost = 0;
-			const equipUpgradeCost = 250;
-			if (x.maxTroops < 50) {
-				cost -= 5000 + (((50 - x.maxTroops) /10) * equipUpgradeCost * (x.equip + x.commissars + x.cyber + x.SF));
-			}
+			if (type !== "bots") {
+				if (x.maxTroops < 50) {
+					cost -= 5000 + (((50 - x.maxTroops) /10) * equipUpgradeCost * (x.equip + x.commissars + x.cyber + x.SF));
+				}
 
-			if (x.commissars < 2) {
-				cost -= (equipUpgradeCost * x.maxTroops + 1000) * (2 - x.commissars);
-			}
-			if ((V.prostheticsUpgrade >= 2 || V.researchLab.advCombatPLimb === 1) && x.cyber === 0) {
-				cost -= equipUpgradeCost * x.maxTroops + 2000;
-			}
-			if (x.medics === 0) {
-				cost -= equipUpgradeCost * x.maxTroops + 1000;
-			}
-			if (V.SF.Toggle && V.SF.Active >= 1 && x.SF === 0) {
-				cost -= equipUpgradeCost * x.maxTroops + 5000;
+				if (x.commissars < 2) {
+					cost -= (equipUpgradeCost * x.maxTroops + 1000) * (2 - x.commissars);
+				}
+				if ((V.prostheticsUpgrade >= 2 || V.researchLab.advCombatPLimb === 1) && x.cyber === 0) {
+					cost -= equipUpgradeCost * x.maxTroops + 2000;
+				}
+				if (x.medics === 0) {
+					cost -= equipUpgradeCost * x.maxTroops + 1000;
+				}
+				if (V.SF.Toggle && V.SF.Active >= 1 && x.SF === 0) {
+					cost -= equipUpgradeCost * x.maxTroops + 5000;
+				}
+			} else {
+				if (unit.maxTroops < 80) {
+					cost -= 5000 * (80 - unit.maxTroops);
+				} else if (V.SF.Toggle && V.SF.Active >= 1 && unit.maxTroops < 100 && V.SecExp.edicts.SFSupportLevel >= 1) {
+					cost -= 5000 + 10 * secBotsUpgradeCost * unit.equip * (100 - unit.maxTroops);
+				}
 			}
 			if (x.equip < 3) {
 				cost -= (equipUpgradeCost * x.maxTroops + 1000) * (3 - x.equip);
@@ -101,7 +120,9 @@ App.SecExp.unit = (function() {
 		}
 
 		let newUnit = {
-			ID: -1, equip: 0, active: 1, isDeployed: 0, maxTroops: 30, troops: 30
+			equip: 0, active: 1,
+			maxTroops: 30, troops: 30,
+			ID: -1, isDeployed: 0
 		};
 		if (type !== "bots") {
 			V.SecExp.units[type].created++;
@@ -310,9 +331,9 @@ App.SecExp.unit = (function() {
 			}
 		} else {
 			if (brief === 0) {
-				App.UI.DOM.appendNewElement("span", el,`The drone unit is made up of ${input.troops} drones. All of which are assembled in an ordered formation in front of you, absolutely silent and ready to receive their orders. `);
+				el.append(`The drone unit is made up of ${input.troops} drones. All of which are assembled in an ordered formation in front of you, absolutely silent and ready to receive their orders. `);
 			} else {
-				App.UI.DOM.appendNewElement("span", el,`Drone squad. `);
+				el.append(`Drone squad. `);
 			}
 		}
 
@@ -402,7 +423,7 @@ App.SecExp.unit = (function() {
 					el.append(`The unit has "advisors" from ${V.SF.Lower} that will help the squad remain tactically aware and active. `);
 				}
 			} else {
-				App.UI.DOM.appendNewElement("span", el, `Training: `);
+				el.append(`Training: `);
 				if (input.training <= 33) {
 					el.append(`low. `);
 				} else if(input.training <= 66) {
@@ -423,9 +444,10 @@ App.SecExp.unit = (function() {
 				} else {
 					el.append(`fanatical. `);
 				}
+				App.UI.DOM.appendNewElement("div", el);
 
 				if (jsDef(input.cyber) && input.cyber > 0) {
-					App.UI.DOM.appendNewElement("div", el, `Cyberaugmentations applied. `);
+					el.append(`Cyberaugmentations applied. `);
 				}
 				if (jsDef(input.medics) && input.medics > 0) {
 					el.append(`Medical squad attached. `);
@@ -483,7 +505,7 @@ App.SecExp.unit = (function() {
 			const expLoss = (squad.troops - oldTroops) / squad.troops;
 			squad.training -= squad.training * expLoss;
 		} else {
-			cashX(-((squad.maxTroops - squad.troops) * 500), "securityExpansion");
+			cashX(-((squad.maxTroops - squad.troops) * secBotsCost), "securityExpansion");
 			squad.troops = squad.maxTroops;
 		}
 	}
@@ -22,39 +25,55 @@ App.SecExp.unit = (function() {
 
 	/** Creates a bulk upgrade link for the unit that is passed.
-	 * @param {object} [unit] the unit to be checked.
+	 * @param {object} [unit] the unit to be checked
+	 * @param {string} [type] the type of unit to be checked
 	 */